@@ -27,6 +27,10 @@
  * See the ChangeLog files for a list of changes. 
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "gedit-encodings-dialog.h"
 
 #include <glade/glade-xml.h>
@@ -235,7 +239,8 @@ get_encodings_dialog (GeditPreferencesDialog *dlg)
 	    !dialog->encodings_treeview) 
 	{
 		g_warning (
-			_("Could not find the required widgets inside gedit-encodings-dialog.glade2.\n"));
+			_("Could not find the required widgets inside %s."),
+			"gedit-encodings-dialog.glade2");
 		g_object_unref (gui);
 		return NULL;
 	}
