@@ -28,19 +28,32 @@
  * See the ChangeLog files for a list of changes. 
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <libgnomeui/libgnomeui.h>
 #include <glib/gunicode.h>
 #include <libgnome/gnome-i18n.h>
-#include <libgnomevfs/gnome-vfs-uri.h>
+#include <libgnomevfs/gnome-vfs.h>
+#include <eel/eel-vfs-extensions.h>
+#include <eel/eel-string.h>
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 
 #include <string.h>
 
 #include "gedit-utils.h"
 #include "gedit2.h"
 #include "bonobo-mdi.h"
-#include "gnome-vfs-helpers.h"
 #include "gedit-document.h"
+#include "gedit-prefs-manager.h"
+#include "gedit-encodings.h"
+#include "gedit-debug.h"
+
+#define STDIN_DELAY_MICROSECONDS 100000
 
 /* =================================================== */
 /* Flash */
@@ -72,7 +85,7 @@ remove_message_timeout (MessageInfo * mi)
 	bonobo_ui_component_set_status (ui_component, " ", NULL);
 
 	g_signal_handler_disconnect (G_OBJECT (mi->win), mi->handlerid);
-
+	
 	g_free (mi);
 	current_mi = NULL;
 
@@ -87,6 +100,9 @@ remove_timeout_cb (GtkWidget *win, MessageInfo *mi)
 {
  	gtk_timeout_remove (mi->timeoutid);
   	g_free (mi);
+
+	if (mi == current_mi)
+	       	current_mi = NULL;
 }
 
 static const guint32 flash_length = 3000; /* 3 seconds, I hope */
@@ -184,40 +200,16 @@ gedit_utils_flash_va (gchar *format, ...)
 	g_free (msg);
 }
 
-static gboolean
-istr_has_prefix (const char *haystack, const char *needle)
-{
-	const char *h, *n;
-	char hc, nc;
-
-	/* Eat one character at a time. */
-	h = haystack == NULL ? "" : haystack;
-	n = needle == NULL ? "" : needle;
-	do {
-		if (*n == '\0') {
-			return TRUE;
-		}
-		if (*h == '\0') {
-			return FALSE;
-		}
-		hc = *h++;
-		nc = *n++;
-		hc = g_ascii_tolower (hc);
-		nc = g_ascii_tolower (nc);
-	} while (hc == nc);
-	return FALSE;
-}
-
 gboolean
 gedit_utils_uri_has_file_scheme (const gchar *uri)
 {
 	gchar* canonical_uri = NULL;
 	gboolean res;
 	
-	canonical_uri = gnome_vfs_x_make_uri_canonical (uri);
+	canonical_uri = eel_make_uri_canonical (uri);
 	g_return_val_if_fail (canonical_uri != NULL, FALSE);
 
-	res = istr_has_prefix (canonical_uri, "file");
+	res = eel_istr_has_prefix (canonical_uri, "file:");
 	
 	g_free (canonical_uri);
 
@@ -228,16 +220,34 @@ gboolean
 gedit_utils_is_uri_read_only (const gchar* uri)
 {
 	gchar* file_uri = NULL;
+	gchar* canonical_uri = NULL;
+
 	gint res;
 
+	g_return_val_if_fail (uri != NULL, TRUE);
+	
+	gedit_debug (DEBUG_FILE, "URI: %s", uri);
+
 	/* FIXME: all remote files are marked as readonly */
 	if (!gedit_utils_uri_has_file_scheme (uri))
 		return TRUE;
 			
-	file_uri = gnome_vfs_x_format_uri_for_display (uri);
-		
+	canonical_uri = eel_make_uri_canonical (uri);
+	g_return_val_if_fail (canonical_uri != NULL, TRUE);
+	
+	gedit_debug (DEBUG_FILE, "CANONICAL URI: %s", canonical_uri);
+
+	file_uri = gnome_vfs_get_local_path_from_uri (canonical_uri);
+	if (file_uri == NULL)
+	{
+		gedit_debug (DEBUG_FILE, "FILE URI: NULL");
+
+		return TRUE;
+	}
+	
 	res = access (file_uri, W_OK);
 
+	g_free (canonical_uri);
 	g_free (file_uri);
 
 	return res;	
@@ -365,7 +375,7 @@ g_utf8_strcasestr(const gchar *haystack, const gchar *needle)
 	/* Not so efficient */
 	while (*p)
 	{
-		if ((memcmp (caseless_haystack, caseless_needle, needle_len) == 0))
+		if ((strncmp (caseless_haystack, caseless_needle, needle_len) == 0))
 		{
 			ret = p;
 			goto finally_1;
@@ -392,7 +402,7 @@ finally_1:
  * n: len of the string in bytes
  */
 gboolean 
-g_uft8_caselessnmatch (const char *s1, const char *s2, gssize n1, gssize n2)
+g_utf8_caselessnmatch (const char *s1, const char *s2, gssize n1, gssize n2)
 {
 	gchar *casefold;
 	gchar *normalized_s1;
@@ -400,7 +410,7 @@ g_uft8_caselessnmatch (const char *s1, const char *s2, gssize n1, gssize n2)
 	gint len_s1;
 	gint len_s2;
 	gboolean ret = FALSE;
-	
+
 	g_return_val_if_fail (s1 != NULL, FALSE);
 	g_return_val_if_fail (s2 != NULL, FALSE);
 	g_return_val_if_fail (n1 > 0, FALSE);
@@ -417,10 +427,10 @@ g_uft8_caselessnmatch (const char *s1, const char *s2, gssize n1, gssize n2)
 	len_s1 = strlen (normalized_s1);
 	len_s2 = strlen (normalized_s2);
 
-	if (len_s1 != len_s2)
+	if (len_s1 < len_s2)
 		goto finally_2;
 
-	ret = (memcmp (normalized_s1, normalized_s2, len_s1) == 0);
+	ret = (strncmp (normalized_s1, normalized_s2, len_s2) == 0);
 	
 finally_2:
 	g_free (normalized_s1);
@@ -526,7 +536,7 @@ lines_match (const GtkTextIter *start,
       /* If it's not the first line, we have to match from the
        * start of the line.
        */
-      if (g_uft8_caselessnmatch (line_text, *lines, strlen (line_text), strlen (*lines)) == 0)
+      if (g_utf8_caselessnmatch (line_text, *lines, strlen (line_text), strlen (*lines)))
         found = line_text;
       else
         found = NULL;
@@ -572,7 +582,6 @@ lines_match (const GtkTextIter *start,
    */
   return lines_match (&next, lines, visible_only, slice, NULL, match_end);
 }
-
 /* strsplit () that retains the delimiter as part of the string. */
 static gchar **
 strbreakup (const char *string,
@@ -753,55 +762,6 @@ gedit_text_iter_forward_search (const GtkTextIter *iter,
  * ERROR REPORTING CODE
  ************************************************************/
 
-/* Note: eel_string_ellipsize_* that use a length in pixels
- * rather than characters can be found in eel_gdk_extensions.h
- * 
- */
-gchar *
-gedit_utils_str_middle_truncate (const gchar *string, guint truncate_length)
-{
-	gchar *truncated;
-	guint length;
-	guint num_left_chars;
-	guint num_right_chars;
-
-	const gchar delimter[] = "...";
-	const guint delimter_length = strlen (delimter);
-	const guint min_truncate_length = delimter_length + 2;
-
-	if (string == NULL) {
-		return NULL;
-	}
-
-	/* It doesn't make sense to truncate strings to less than
-	 * the size of the delimiter plus 2 characters (one on each
-	 * side)
-	 */
-	if (truncate_length < min_truncate_length) {
-		return g_strdup (string);
-	}
-
-	length = strlen (string);
-
-	/* Make sure the string is not already small enough. */
-	if (length <= truncate_length) {
-		return g_strdup (string);
-	}
-
-	/* Find the 'middle' where the truncation will occur. */
-	num_left_chars = (truncate_length - delimter_length) / 2;
-	num_right_chars = truncate_length - num_left_chars - delimter_length + 1;
-
-	truncated = g_new (char, truncate_length + 1);
-
-	strncpy (truncated, string, num_left_chars);
-	strncpy (truncated + num_left_chars, delimter, delimter_length);
-	strncpy (truncated + num_left_chars + delimter_length, string + length - num_right_chars + 1, 
-		 num_right_chars);
-	
-	return truncated;
-}
-
 #define MAX_URI_IN_DIALOG_LENGTH 50
 
 void
@@ -822,13 +782,13 @@ gedit_utils_error_reporting_loading_file (
 	g_return_if_fail (uri != NULL);
 	g_return_if_fail (error != NULL);
 	
-	full_formatted_uri = gnome_vfs_x_format_uri_for_display (uri);
+	full_formatted_uri = eel_format_uri_for_display (uri);
 
 	/* Truncate the URI so it doesn't get insanely wide. Note that even
 	 * though the dialog uses wrapped text, if the URI doesn't contain
 	 * white space then the text-wrapping code is too stupid to wrap it.
 	 */
-        uri_for_display = gedit_utils_str_middle_truncate (full_formatted_uri, 
+        uri_for_display = eel_str_middle_truncate (full_formatted_uri, 
 			MAX_URI_IN_DIALOG_LENGTH);
 	g_free (full_formatted_uri);
 
@@ -849,22 +809,23 @@ gedit_utils_error_reporting_loading_file (
 			break;			 
 
 		case GNOME_VFS_ERROR_NOT_SUPPORTED:
-			scheme_string = gnome_vfs_x_uri_get_scheme (uri);
+			scheme_string = eel_uri_get_scheme (uri);
                 
-			if (scheme_string != NULL)
+			if ((scheme_string != NULL) && g_utf8_validate (scheme_string, -1, NULL))
 			{
 				error_message = g_strdup_printf (
 					_("Could not open the file \"%s\" because "
 					  "gedit cannot handle %s: locations."),
                                 	uri_for_display, scheme_string);
-
-				g_free (scheme_string);
 			}
 			else
 				error_message = g_strdup_printf (
 					_("Could not open the file \"%s\""),
                                 	uri_for_display);
 	
+			if (scheme_string != NULL)
+				g_free (scheme_string);
+
         	        break;
 				
 		case GNOME_VFS_ERROR_WRONG_FORMAT:
@@ -923,17 +884,32 @@ gedit_utils_error_reporting_loading_file (
 		 	* But this case is also hit for legitimate web addresses when
 		 	* the proxy is set up wrong.
 		 	*/
-			vfs_uri = gnome_vfs_uri_new (uri);
-                	error_message = g_strdup_printf (
-				_("Could not open the file \"%s\" because no host \"%s\" " 
-				  "could be found.\n\n"
-                		  "Please, check that you typed the location correctly "
-				  "and that your proxy settings are correct and then "
-				  "try again."),
-				uri_for_display,
-				gnome_vfs_uri_get_host_name (vfs_uri));
+			{
+				gchar *host_name;
+				vfs_uri = gnome_vfs_uri_new (uri);
+				
+				if (vfs_uri == NULL)
+					host_name = g_strdup ("XXX");
+				else
+				{
+					host_name = eel_make_valid_utf8 (
+							gnome_vfs_uri_get_host_name (vfs_uri));
+
+					gnome_vfs_uri_unref (vfs_uri);		
+				}
+				
+                		error_message = g_strdup_printf (
+					_("Could not open the file \"%s\" because no host \"%s\" " 
+				  	  "could be found.\n\n"
+                		  	  "Please, check that you typed the location correctly "
+				  	  "and that your proxy settings are correct and then "
+				  	  "try again."),
+					  uri_for_display,
+					  host_name);
+
+				g_free (host_name);
+			}
 
-			gnome_vfs_uri_unref (vfs_uri);
 			break;
 
 		case GNOME_VFS_ERROR_INVALID_HOST_NAME:
@@ -1037,13 +1013,13 @@ gedit_utils_error_reporting_saving_file (
 	g_return_if_fail (uri != NULL);
 	g_return_if_fail (error != NULL);
 	
-	full_formatted_uri = gnome_vfs_x_format_uri_for_display (uri);
+	full_formatted_uri = eel_format_uri_for_display (uri);
 
 	/* Truncate the URI so it doesn't get insanely wide. Note that even
 	 * though the dialog uses wrapped text, if the URI doesn't contain
 	 * white space then the text-wrapping code is too stupid to wrap it.
 	 */
-        uri_for_display = gedit_utils_str_middle_truncate (full_formatted_uri, 
+        uri_for_display = eel_str_middle_truncate (full_formatted_uri, 
 			MAX_URI_IN_DIALOG_LENGTH);
 	g_free (full_formatted_uri);
 
@@ -1092,13 +1068,13 @@ gedit_utils_error_reporting_reverting_file (
 	g_return_if_fail (uri != NULL);
 	g_return_if_fail (error != NULL);
 	
-	full_formatted_uri = gnome_vfs_x_format_uri_for_display (uri);
+	full_formatted_uri = eel_format_uri_for_display (uri);
 
 	/* Truncate the URI so it doesn't get insanely wide. Note that even
 	 * though the dialog uses wrapped text, if the URI doesn't contain
 	 * white space then the text-wrapping code is too stupid to wrap it.
 	 */
-        uri_for_display = gedit_utils_str_middle_truncate (full_formatted_uri, 
+        uri_for_display = eel_str_middle_truncate (full_formatted_uri, 
 			MAX_URI_IN_DIALOG_LENGTH);
 	g_free (full_formatted_uri);
 
@@ -1119,22 +1095,23 @@ gedit_utils_error_reporting_reverting_file (
 			break;			 
 
 		case GNOME_VFS_ERROR_NOT_SUPPORTED:
-			scheme_string = gnome_vfs_x_uri_get_scheme (uri);
+			scheme_string = eel_uri_get_scheme (uri);
                 
-			if (scheme_string != NULL)
+			if ((scheme_string != NULL) && g_utf8_validate (scheme_string, -1, NULL))
 			{
 				error_message = g_strdup_printf (
 					_("Could not revert the file \"%s\" because "
 					  "gedit cannot handle %s: locations."),
                                 	uri_for_display, scheme_string);
-
-				g_free (scheme_string);
 			}
 			else
 				error_message = g_strdup_printf (
 					_("Could not revert the file \"%s\"."),
                                 	uri_for_display);
 	
+			if (scheme_string != NULL)
+				g_free (scheme_string);
+
         	        break;
 				
 		case GNOME_VFS_ERROR_WRONG_FORMAT:
@@ -1179,16 +1156,31 @@ gedit_utils_error_reporting_reverting_file (
 		 	* But this case is also hit for legitimate web addresses when
 		 	* the proxy is set up wrong.
 		 	*/
-			vfs_uri = gnome_vfs_uri_new (uri);
-                	error_message = g_strdup_printf (
-				_("Could not revert the file \"%s\" because no host \"%s\" " 
-				  "could be found.\n\n"
-                		  "Please, check that your proxy settings are correct and "
-				  "try again."),
-				uri_for_display,
-				gnome_vfs_uri_get_host_name (vfs_uri));
+			{
+				gchar *host_name;
+				vfs_uri = gnome_vfs_uri_new (uri);
+				
+				if (vfs_uri == NULL)
+					host_name = g_strdup ("XXX");
+				else
+				{
+					host_name = eel_make_valid_utf8 (
+							gnome_vfs_uri_get_host_name (vfs_uri));
+
+					gnome_vfs_uri_unref (vfs_uri);		
+				}
+				
+				error_message = g_strdup_printf (
+					_("Could not revert the file \"%s\" because no host \"%s\" " 
+				  	  "could be found.\n\n"
+                		  	  "Please, check that your proxy settings are correct and "
+				  	  "try again."),
+					  uri_for_display,
+					  host_name);
+
+				g_free (host_name);
+			}
 
-			gnome_vfs_uri_unref (vfs_uri);
 			break;
             
 		case GNOME_VFS_ERROR_HOST_HAS_NO_ADDRESS:
@@ -1278,3 +1270,436 @@ gedit_utils_error_reporting_reverting_file (
 	g_free (error_message);
 }
 
+void
+gedit_utils_error_reporting_creating_file (
+		const gchar *uri,
+		gint error_code,
+		GtkWindow *parent)
+{
+	gchar *error_message;
+	gchar *full_formatted_uri;
+       	gchar *uri_for_display	;
+	
+	GtkWidget *dialog;
+
+	g_return_if_fail (uri != NULL);
+	
+	full_formatted_uri = eel_format_uri_for_display (uri);
+
+	/* Truncate the URI so it doesn't get insanely wide. Note that even
+	 * though the dialog uses wrapped text, if the URI doesn't contain
+	 * white space then the text-wrapping code is too stupid to wrap it.
+	 */
+        uri_for_display = eel_str_middle_truncate (full_formatted_uri, 
+			MAX_URI_IN_DIALOG_LENGTH);
+	g_free (full_formatted_uri);
+
+	switch (error_code)
+	{
+		case EEXIST: 
+			error_message = g_strdup_printf (_("The file \"%s\" already exists."),
+				uri_for_display);
+			break;
+
+		case EISDIR:
+			error_message = g_strdup_printf (
+				_("\"%s\" is a directory.\n\n"
+				  "Please, check that you typed the location correctly."),
+                         	uri_for_display);
+                	break;
+
+		case EACCES:
+		case EROFS:
+		case ETXTBSY:
+			error_message = g_strdup_printf (
+				_("Cannot create the file \"%s\".\n\n"
+				  "Make sure you have the appropriate write permissions."),
+				uri_for_display);
+			break;
+
+		case ENAMETOOLONG:
+			error_message = g_strdup_printf (
+				_("Cannot create the file \"%s\".\n\n"
+				  "The file name is too long."),
+				uri_for_display);
+			break;
+		
+		case ENOENT:
+			error_message = g_strdup_printf (
+				_("Cannot create the file \"%s\".\n\n"
+				  "A directory component in the file name does not exist or "
+		                  "is a dangling symbolic link."),
+				uri_for_display);	
+			break;
+
+
+		case ENOSPC:
+			error_message = g_strdup_printf (
+				 _("There is not enough disk space to create the file \"%s\".\n\n"
+				   "Please free some disk space and try again."),
+				 uri_for_display);
+			break;
+
+		default:
+			error_message = g_strdup_printf (_("Could not create the file \"%s\"."),
+							 uri_for_display);
+	}
+	
+	dialog = gtk_message_dialog_new (
+			parent,
+			GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+		   	GTK_MESSAGE_ERROR,
+		   	GTK_BUTTONS_OK,
+			error_message);
+			
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+
+	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+
+	g_free (uri_for_display);
+	g_free (error_message);
+}
+
+
+/*
+ * gedit_utils_set_atk_name_description
+ * @widget : The Gtk widget for which name/description to be set
+ * @name : Atk name string
+ * @description : Atk description string
+ * Description : This function sets up name and description
+ * for a specified gtk widget.
+ */
+
+void
+gedit_utils_set_atk_name_description (	GtkWidget *widget, 
+					const gchar *name,
+					const gchar *description)
+{
+	AtkObject *aobj;
+
+	aobj = gtk_widget_get_accessible (widget);
+
+	if (!(GTK_IS_ACCESSIBLE (aobj)))
+		return;
+
+	if(name)
+		atk_object_set_name (aobj, name);
+
+	if(description)
+		atk_object_set_description (aobj, description);
+}
+/*
+ * gedit_set_atk__relation
+ * @obj1,@obj2 : specified widgets.
+ * @rel_type : the type of relation to set up.
+ * Description : This function establishes atk relation
+ * between 2 specified widgets.
+ */
+void
+gedit_utils_set_atk_relation (	GtkWidget *obj1, 
+				GtkWidget *obj2, 
+				AtkRelationType rel_type )
+{
+	AtkObject *atk_obj1, *atk_obj2;
+	AtkRelationSet *relation_set;
+	AtkObject *targets[1];
+	AtkRelation *relation;
+
+	atk_obj1 = gtk_widget_get_accessible (obj1);
+	atk_obj2 = gtk_widget_get_accessible (obj2);
+
+	if (!(GTK_IS_ACCESSIBLE (atk_obj1)) || !(GTK_IS_ACCESSIBLE (atk_obj2)))
+		return;
+
+	relation_set = atk_object_ref_relation_set (atk_obj1);
+	targets[0] = atk_obj2;
+
+	relation = atk_relation_new (targets, 1, rel_type);
+	atk_relation_set_add (relation_set, relation);
+
+	g_object_unref (G_OBJECT (relation));
+}
+
+gboolean
+gedit_utils_uri_exists (const gchar* text_uri)
+{
+	GnomeVFSURI *uri;
+	gboolean res;
+		
+	g_return_val_if_fail (text_uri != NULL, FALSE);
+	
+	gedit_debug (DEBUG_FILE, "text_uri: %s", text_uri);
+
+	uri = gnome_vfs_uri_new (text_uri);
+	g_return_val_if_fail (uri != NULL, FALSE);
+
+	res = gnome_vfs_uri_exists (uri);
+
+	gnome_vfs_uri_unref (uri);
+
+	gedit_debug (DEBUG_FILE, res ? "TRUE" : "FALSE");
+
+	return res;
+}
+
+gchar *
+gedit_utils_convert_search_text (const gchar *text)
+{
+	GString *str;
+	gint length;
+	gboolean drop_prev = FALSE;
+	const gchar *cur;
+	const gchar *end;
+	const gchar *prev;
+
+	g_return_val_if_fail (text != NULL, NULL);
+
+	length = strlen (text);
+
+	str = g_string_new ("");
+
+	cur = text;
+	end = text + length;
+	prev = NULL;
+	while (cur != end) {
+		const gchar *next;
+		next = g_utf8_next_char (cur);
+
+		if (prev && (*prev == '\\')) {
+			switch (*cur) {
+				case 'n':
+					str = g_string_append (str, "\n");
+				break;
+				case 'r':
+					str = g_string_append (str, "\r");
+				break;
+				case 't':
+					str = g_string_append (str, "\t");
+				break;
+				case '\\':
+					str = g_string_append (str, "\\");
+					drop_prev = TRUE;
+				break;
+				default:
+					str = g_string_append (str, "\\");
+					str = g_string_append (str, cur);
+				break;
+			}
+		} else if (*cur != '\\') {
+			str = g_string_append_len (str, cur, next - cur);
+		}
+
+		if (!drop_prev)
+			prev = cur;
+		else {
+			prev = NULL;
+			drop_prev = FALSE;
+		}
+
+		cur = next;
+	}
+
+	return g_string_free (str, FALSE);
+}
+
+gboolean
+gedit_utils_create_empty_file (const gchar *uri)
+{
+	gchar *canonical_uri;
+	gchar *filename;
+	int fd;
+	
+	g_return_val_if_fail (uri != NULL, FALSE);
+	
+	/* Get filename from uri */
+	if (!gedit_utils_uri_has_file_scheme (uri))
+		return FALSE;
+			
+	canonical_uri = eel_make_uri_canonical (uri);
+	g_return_val_if_fail (canonical_uri != NULL, FALSE);
+	
+	gedit_debug (DEBUG_FILE, "CANONICAL URI: %s", canonical_uri);
+
+	filename = gnome_vfs_get_local_path_from_uri (canonical_uri);
+	g_free (canonical_uri);
+	
+	if (filename == NULL)
+	{
+		gedit_debug (DEBUG_FILE, "FILENAME: NULL");
+
+		return FALSE;
+	}
+	
+	fd = open (filename, O_CREAT | O_EXCL | O_WRONLY, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
+
+	g_free (filename);
+	
+	if (fd == -1)
+		return FALSE;
+	
+	return (close (fd) == 0);
+}
+
+
+
+
+#define GEDIT_STDIN_BUFSIZE 1024
+
+gchar *
+gedit_utils_get_stdin (void)
+{
+	GString * file_contents;
+	gchar *tmp_buf = NULL;
+	guint buffer_length;
+	GnomeVFSResult	res;
+	fd_set rfds;
+	struct timeval tv;
+	
+	FD_ZERO (&rfds);
+	FD_SET (0, &rfds);
+
+	/* wait for 1/4 of a second */
+	tv.tv_sec = 0;
+	tv.tv_usec = STDIN_DELAY_MICROSECONDS;
+
+	if (select (1, &rfds, NULL, NULL, &tv) != 1)
+		return NULL;
+
+	tmp_buf = g_new0 (gchar, GEDIT_STDIN_BUFSIZE + 1);
+	g_return_val_if_fail (tmp_buf != NULL, FALSE);
+
+	file_contents = g_string_new (NULL);
+	
+	while (feof (stdin) == 0)
+	{
+		buffer_length = fread (tmp_buf, 1, GEDIT_STDIN_BUFSIZE, stdin);
+		tmp_buf [buffer_length] = '\0';
+		g_string_append (file_contents, tmp_buf);
+
+		if (ferror (stdin) != 0)
+		{
+			res = gnome_vfs_result_from_errno (); 
+		
+			g_free (tmp_buf);
+			g_string_free (file_contents, TRUE);
+			return NULL;
+		}
+	}
+
+	fclose (stdin);
+
+	return g_string_free (file_contents, FALSE);
+}
+
+
+
+static gchar *
+gedit_utils_convert_to_utf8_from_charset (const gchar *content,
+					  gsize len,
+					  const gchar *charset)
+{
+	gchar *utf8_content = NULL;
+	GError *conv_error = NULL;
+	gchar* converted_contents = NULL;
+	gsize bytes_written;
+
+	g_return_val_if_fail (content != NULL, NULL);
+
+	gedit_debug (DEBUG_UTILS, "Trying to convert from %s to UTF-8", charset);
+			
+	converted_contents = g_convert (content, len, "UTF-8",
+					charset, NULL, &bytes_written,
+					&conv_error); 
+						
+	if ((conv_error != NULL) || 
+	    !g_utf8_validate (converted_contents, bytes_written, NULL))		
+	{
+		gedit_debug (DEBUG_UTILS, "Couldn't convert from %s to UTF-8.",
+			     charset);
+				
+		if (converted_contents != NULL)
+			g_free (converted_contents);
+
+		if (conv_error != NULL)
+		{
+			g_error_free (conv_error);
+			conv_error = NULL;
+		}
+
+		utf8_content = NULL;
+	} else {
+		gedit_debug (DEBUG_UTILS,
+			"Converted from %s to UTF-8.",
+			charset);
+		utf8_content = converted_contents;
+	}
+
+	return utf8_content;
+}
+
+gchar *
+gedit_utils_convert_to_utf8 (const gchar *content, gsize len,
+			     gchar **encoding_used)
+{
+	GSList *encodings = NULL;
+	GSList *start;
+	const gchar *locale_charset;
+
+	g_return_val_if_fail (!g_utf8_validate (content, len, NULL), 
+			g_strndup (content, len < 0 ? strlen (content) : len));
+
+	encodings = gedit_prefs_manager_get_encodings ();
+
+	if (g_get_charset (&locale_charset) == FALSE) 
+	{
+		const GeditEncoding *locale_encoding;
+
+		/* not using a UTF-8 locale, so try converting
+		 * from that first */
+		if (locale_charset != NULL)
+		{
+			locale_encoding = gedit_encoding_get_from_charset (locale_charset);
+			encodings = g_slist_prepend (encodings,
+						(gpointer)locale_encoding);
+		}
+	}
+
+	start = encodings;
+
+	while (encodings != NULL) 
+	{
+		GeditEncoding *enc;
+		const gchar *charset;
+		gchar *utf8_content;
+
+		enc = (GeditEncoding *)encodings->data;
+
+		charset = gedit_encoding_get_charset (enc);
+
+		gedit_debug (DEBUG_UTILS, "Trying to convert %ld bytes of data into UTF-8.", len);
+		fflush (stdout);
+		utf8_content = gedit_utils_convert_to_utf8_from_charset
+							(content, len, charset);
+
+		if (utf8_content != NULL) {
+			if (encoding_used != NULL)
+			{
+				if (*encoding_used != NULL)
+					g_free (*encoding_used);
+				
+				*encoding_used = g_strdup (charset);
+			}
+
+			return utf8_content;
+		}
+
+		encodings = encodings->next;
+	}
+
+	g_slist_free (start);
+	
+	return NULL;
+}
